@@ -17,7 +17,7 @@
 #include <QTextStream>
 
 // KDE
-#include <qtest.h>
+#include <QTest>
 
 using namespace Konsole;
 
